@@ -116,23 +116,32 @@ class JobView(graphlab.canvas.views.base.BaseView):
                 log_data['data'] = self.__cache_logs
         elif isinstance(job.environment, graphlab.deploy.environment.EC2):
             if self.__cache_logs is None:
-                if not isinstance(log_file_path, tuple):
+                if not isinstance(log_file_path, list):
                     log_data['data'] = log_file_path
                     return log_data
             
-                log_data['url'] = log_file_path[0]
-                # load max of 1mb log file to python
-                if log_file_path[1] > 1048576L:
-                    log_data['data'] = "Log file too large to be displayed. Please use the link to download the log file directly"
-                    return log_data
+                if len(log_file_path) == 1:
+                    # only a single log file, so try to display it
+                    log_file = log_file_path[0]
+                    log_data['url'] = log_file[0]
+                    # load max of 1mb log file to python
+                    if log_file[1] > 1048576L:
+                        log_data['data'] = "Log file too large to be displayed. Please use the link to download the log file directly"
+                        return log_data
                     
-                data = self.__load_s3_log_file(log_file_path[0])
-                log_data['data'] = data
-                status = job.get_status()
-                if status == "Completed" or status == "Failed":
-                    self.__cache_logs = data
+                    data = self.__load_s3_log_file(log_file[0])
+                    log_data['data'] = data
+                    status = job.get_status()
+                    if status == "Completed" or status == "Failed":
+                        self.__cache_logs = data
+                else:
+                    # a list of log files, only show download links
+                    log_data['url'] = ''
+                    log_data['data'] = [log_file[0] for log_file in log_file_path] 
+                    # always cache this since its only urls
+                    self.__cache_logs = log_data['data']
             else:
-                log_data['url'] = log_file_path[0]
+                log_data['url'] = log_file_path[0][0] if len(log_file_path) == 1 else ''
                 log_data['data'] = self.__cache_logs
         return log_data 
 
