@@ -25,14 +25,14 @@ class PredictiveService(graphlab.canvas.views.base.BaseView):
         return data
 
     def __get_ps_info(self):
-        name = self.obj 
+        name = self.obj
         ps = self.__load_ps_object()
         # return to session page if loaded ps none
         if ps is None:
-            return {'deleted': True} 
+            return {'deleted': True}
 
         ps_info = {}
-        
+
         # get deployed and pending predictive objects
         ps_info['deployed'] = ps.deployed_predictive_objects
         ps_info['pending'] = ps.pending_changes
@@ -53,7 +53,7 @@ class PredictiveService(graphlab.canvas.views.base.BaseView):
 
     def __get_ps_status(self, url, handler):
         ps = self.__load_ps_object()
-        
+
         # get status list of all instances
         status_info = {}
         try:
@@ -68,18 +68,10 @@ class PredictiveService(graphlab.canvas.views.base.BaseView):
             status_info = {}
 
         # get metrics url
-        metrics_url = ''
-        try:
-            raw_text = ps.get_metrics_url()
-            parsed = json.loads(str(raw_text))
-            if isinstance(parsed, dict):
-                if 'url' in parsed:
-                    metrics_url = parsed['url']
-        except Exception as e:
-            metrics_url = ''
+        metrics_url = ps.get_metrics_url()
 
         handler.write({'ps_status': status_info, 'ps_metrics_url': metrics_url})
-    
+
     def __get_metrics_data(self, sf, x, y, time_frame):
         # get x, y values of the metrics
         xList = list(sf[x])
@@ -101,11 +93,11 @@ class PredictiveService(graphlab.canvas.views.base.BaseView):
         ps = self.__load_ps_object()
         # get metrics
         (time_frame, num_requests, latency, hosts, cache) = ps.get_metrics()
-        # initialize empty data array 
-        num_requests_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}] 
-        latency_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}] 
-        hosts_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}] 
-        cache_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}] 
+        # initialize empty data array
+        num_requests_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}]
+        latency_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}]
+        hosts_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}]
+        cache_data = [[], {'minX':0, 'maxX':0, 'minY': 0, 'maxY':0}]
         # add column containing int value of datetime
         if num_requests is not None:
             num_requests = num_requests.add_column(num_requests['num_requests.Timestamp'].astype(int), name='num_requests.time')
@@ -125,7 +117,7 @@ class PredictiveService(graphlab.canvas.views.base.BaseView):
                         "hosts_data" : hosts_data,
                         "cache_data" : cache_data})
 
-            
+
     def __load_ps_object(self):
         """
         Loads the predictive object by name, from the current session.
