@@ -34,10 +34,11 @@ VALID_REGIONS = [r.name for r in boto.ec2.get_regions('ec2')]
 
 # Port 9000 & 9001 - used when running in server mode: only server running with external client.
 # Port 9002 & 9003 - used for metrics (deprecated).
-# Port 9004 - for jobs/tasks.
+# Port 9004 - Commander daemon for jobs/tasks.
 # Port 9005 & 9006 - used by predictive service (webservice & reddis).
+# Port 9007 - Worker daemon for jobs/tasks.
 GRAPHLAB_PORT_MIN_NUM = 9000
-GRAPHLAB_PORT_MAX_NUM = 9006
+GRAPHLAB_PORT_MAX_NUM = 9007
 ADDITIONAL_REDIS_PORT = 19006
 
 # Swallow all boto logging, except critical issues
@@ -209,7 +210,7 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
 
     # Check existing connection
     if glconnect.is_connected():
-        __LOGGER__.warning('You have GraphLab objects instantiated on the current server. If you want to \n'
+        __LOGGER__.error('You have GraphLab objects instantiated on the current server. If you want to \n'
                            'launch a new server, you must first stop the current server connection by running: \n'
                            '\'graphlab.connect.main.stop()\'. Be warned: you will lose these instantiated objects.')
         return
@@ -222,7 +223,7 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
         (client_public_key, client_secret_key) = get_public_secret_key_pair()
         __LOGGER__.debug('Launching server with public key: %s' % server_public_key)
 
-    try: 
+    try:
         server = _Ec2GraphLabServer(instance_type, region, CIDR_rule, auth_token, security_group, tags,
                                 server_public_key, server_secret_key)
 
@@ -454,16 +455,16 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
 
     # Configure the security group
     try:
+        security_group.authorize('tcp', GRAPHLAB_PORT_MIN_NUM, GRAPHLAB_PORT_MAX_NUM, CIDR_rule)
         security_group.authorize('tcp', 443, 443, CIDR_rule)
         security_group.authorize('tcp', 80, 80, CIDR_rule)
         security_group.authorize('tcp', ADDITIONAL_REDIS_PORT, ADDITIONAL_REDIS_PORT, CIDR_rule)
-        security_group.authorize('tcp', GRAPHLAB_PORT_MIN_NUM, GRAPHLAB_PORT_MAX_NUM, CIDR_rule)
     except EC2ResponseError as e:
         if not "<Code>InvalidPermission.Duplicate</Code>" in str(e):
             raise e
 
     # Determine AMI id, as well as engine, os URL, and hash value
-    if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ and 'GRAPHLAB_TEST_HASH_KEY' in os.environ):
+    if ('GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ and 'GRAPHLAB_TEST_HASH_KEY' in os.environ):
         # unit-test mode, don't involve webservice to retrieve AMI, instead use environment variables
         ami_id = os.environ['GRAPHLAB_TEST_AMI_ID']
         engine_url = os.environ['GRAPHLAB_TEST_ENGINE_URL']
@@ -522,7 +523,7 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
     # If this isn't prod, may not to set SSH key for debugging.
     if 'GRAPHLAB_TEST_EC2_KEYPAIR' in os.environ:
         keypair = os.environ['GRAPHLAB_TEST_EC2_KEYPAIR']
-        __LOGGER__.info("UNIT mode, using keypair: '%s' when launching EC2 instance" % (keypair))
+        __LOGGER__.info("Using keypair: '%s' when launching EC2 instance" % (keypair))
         run_instances_args['key_name'] = keypair
 
     # Actually launch the EC2 instance(s) and wait for them to start running.
@@ -555,7 +556,7 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
             except EC2ResponseError as e:
                 # EC2 is eventual consistence so sometimes it complains that it
                 # cannot find the instance, in that case, we will retry
-                __LOGGER__.info("Ignoring EC2ResponseError: %s" % e.message)
+                __LOGGER__.debug("Ignoring EC2ResponseError: %s" % e.message)
 
     except Exception as e:
         if instances:
