@@ -14,12 +14,6 @@ _CURRENT_VERSION = 0
 _format = {_JPG: 0, _PNG: 1, _RAW: 2, _UNDEFINED: 3}
 
 
-def _get_extension():
-    import graphlab.extensions as extensions
-    extensions.list_functions()
-    return extensions
-
-
 class Image(object):
     """
     A class whose objects contains image data, meta-data, and supports useful
@@ -63,7 +57,8 @@ class Image(object):
 
         if (path is not None):
             from graphlab.util import make_internal_url as _make_internal_url
-            img = _get_extension().load_image(_make_internal_url(path), format)
+            import graphlab.extensions as _extensions
+            img = _extensions.load_image(_make_internal_url(path), format)
             for key, value in img.__dict__.iteritems():
                 setattr(self, key, value)
         else:
@@ -154,7 +149,7 @@ class Image(object):
             ie. it is stored as RGBRGBRGB, where each sequence of three values
             represents the Red, Green, and Blue. If the image is grayscale,
             there is only one value per pixel. If the image is RGBA, there are
-            four values per pixel.
+            four values per pixel. The pixels are stored in row-major order.
 
 
         See Also
