@@ -9,6 +9,7 @@ level import of the graphlab package.
 
 """
 
+import graphlab.connect as _mt
 import graphlab.connect.main as glconnect
 from graphlab.data_structures.sframe import SFrame
 from graphlab.data_structures.sarray import SArray
@@ -306,6 +307,8 @@ class SGraph(object):
         | hippo |  0.0   | 0.0  |     0      |
         +-------+--------+------+------------+
         """
+        _mt._get_metric_tracker().track('sgraph.vertices')
+
         return self._vertices
 
     @property
@@ -336,6 +339,8 @@ class SGraph(object):
         |   dog    |  fossa   |    likes     |   equal to   |
         +----------+----------+--------------+--------------+
         """
+        _mt._get_metric_tracker().track('sgraph.edges')
+
         return self._edges
 
     def summary(self):
@@ -360,6 +365,7 @@ class SGraph(object):
         >>> n_edge = g.summary()['num_edges']
         0
         """
+        _mt._get_metric_tracker().track('sgraph.summary')
         ret = self.__proxy__.summary()
         return dict(ret.items())
 
@@ -432,6 +438,8 @@ class SGraph(object):
         |  0   |   M    |
         +------+--------+
         """
+        _mt._get_metric_tracker().track('sgraph.get_vertices')
+
         if not hasattr(ids, '__iter__'):
             ids = [ids]
 
@@ -527,6 +535,8 @@ class SGraph(object):
         |    1     |    2     |  None  |
         +----------+----------+--------+
         """
+        _mt._get_metric_tracker().track('sgraph.get_edges')
+
         if not hasattr(src_ids, '__iter__'):
             src_ids = [src_ids]
         if not hasattr(dst_ids, '__iter__'):
@@ -613,6 +623,8 @@ class SGraph(object):
         >>> sf_vert = SFrame({'id': [0, 1, 2], 'breed':['lab', 'lab', 'vizsla']})
         >>> g = g.add_vertices(sf_vert, vid_field='id')
         """
+        _mt._get_metric_tracker().track('sgraph.add_vertices')
+
         sf = _vertex_data_to_sframe(vertices, vid_field)
 
         with cython_context():
@@ -684,6 +696,8 @@ class SGraph(object):
         >>> sf_edge = SFrame({'source': [0, 1], 'dest': [2, 2]})
         >>> g = g.add_edges(sf_edge, src_field='source', dst_field='dest')
         """
+        _mt._get_metric_tracker().track('sgraph.add_edges')
+
         sf = _edge_data_to_sframe(edges, src_field, dst_field)
 
         with cython_context():
@@ -716,6 +730,8 @@ class SGraph(object):
         >>> fields = g.get_fields()
         ['__id', 'name', '__src_id', '__dst_id', 'frequency']
         """
+        _mt._get_metric_tracker().track('sgraph.get_fields')
+
         return self.get_vertex_fields() + self.get_edge_fields()
 
     def get_vertex_fields(self):
@@ -742,6 +758,8 @@ class SGraph(object):
         >>> fields = g.get_vertex_fields()
         ['__id', 'name']
         """
+        _mt._get_metric_tracker().track('sgraph.')
+
         with cython_context():
             return self.__proxy__.get_vertex_fields()
 
@@ -769,6 +787,8 @@ class SGraph(object):
         >>> fields = g.get_vertex_fields()
         ['__src_id', '__dst_id', 'frequency']
         """
+        _mt._get_metric_tracker().track('sgraph.get_edge_fields')
+
         with cython_context():
             return self.__proxy__.get_edge_fields()
 
@@ -802,6 +822,8 @@ class SGraph(object):
         >>> g = g.add_vertices(verts)
         >>> g2 = g.select_fields(fields=['breed'])
         """
+        _mt._get_metric_tracker().track('sgraph.select_fields')
+
         vfields = self.__proxy__.get_vertex_fields()
         efields = self.__proxy__.get_edge_fields()
         selected_vfields = []
@@ -922,6 +944,7 @@ class SGraph(object):
         Rows: 5
         [4, 1, 1, 1, 4]
         '''
+        _mt._get_metric_tracker().track('sgraph.triple_apply')
 
         assert inspect.isfunction(triple_apply_fn), "Input must be a function"
         if not (type(mutated_fields) is list or type(mutated_fields) is str):
@@ -999,6 +1022,8 @@ class SGraph(object):
 
         >>> g.save('mygraph.json', format='json')
         """
+        _mt._get_metric_tracker().track('sgraph.save')
+
         if format is 'auto':
             if filename.endswith(('.json', '.json.gz')):
                 format = 'json'
@@ -1110,6 +1135,8 @@ class SGraph(object):
         >>> g = sg.add_edges([graphlab.Edge(i, i+1) for i in range(5)])
         >>> g.show(highlight=[2, 3], vlabel='id', arrows=True)
         """
+        _mt._get_metric_tracker().track('sgraph.show')
+
         if isinstance(highlight, SArray):
             # convert to list
             highlight = list(highlight)
@@ -1183,6 +1210,8 @@ class SGraph(object):
                                           full_subgraph=True)
         """
 
+        _mt._get_metric_tracker().track('sgraph.get_neighborhood')
+
         verts = ids
 
         ## find the vertices within radius (and the path edges)
@@ -1269,6 +1298,9 @@ def load_sgraph(filename, format='binary', delimiter='auto'):
     >>> g.save('mygraph')
     >>> g2 = graphlab.load_graph('mygraph')
     """
+
+    _mt._get_metric_tracker().track('sgraph.load_sgraph')
+
     if not format in ['binary', 'snap', 'csv', 'tsv']:
         raise ValueError('Invalid format: %s' % format)
 
