@@ -246,12 +246,14 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         super(Ec2ExecutionEnvironment, self).__init__(ENVIRONMENT.EC2)
 
     @staticmethod
-    def _start_commander_host(environment, credentials):
+    def _start_commander_host(environment, credentials, s3_log_key_prefix):
+        num_children = environment.num_hosts - 1
         user_data = {
-            'daemon': True, 'num_children_host': environment.num_hosts - 1,
-            'aws_access_key': credentials['aws_access_key_id'],
-            'aws_secret_key': credentials['aws_secret_access_key'],
-            'auth_token': ''
+            'auth_token': '', 'aws_access_key': credentials['aws_access_key_id'],
+            'aws_secret_key': credentials['aws_secret_access_key'], 'daemon': True,
+            'commander': True, 'num_children_host': num_children,
+            's3_log_bucket': environment.s3_bucket,
+            's3_log_key_prefix': s3_log_key_prefix
             }
 
         if('GRAPHLAB_TEST_AMI_ID' in _os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in _os.environ):
@@ -266,6 +268,14 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
                                  security_group_name = environment.security_group,
                                  tags = environment.tags, user_data = user_data,
                                  credentials = credentials)
+
+        # Log message explaining the additional hosts will not be launched by us.
+        if num_children == 1:
+            __LOGGER__.info("One host will be launched by %s" % commander.instance_id)
+        elif num_children > 1:
+            __LOGGER__.info("%d hosts will be launched by %s"
+                            % (num_children, commander.instance_id))
+
         return commander
 
     def run_job(self, job, session=None):
@@ -280,15 +290,13 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
 
         s3_log_folder = job.environment.s3_log_folder_path + job.name + '-' + str(_time.time()) + '/'
         s3_state_key = s3_log_folder + 'status'
-        s3_log_key = s3_log_folder + 'log.txt'
-        s3_status_key = s3_log_folder + 'jobstatus.txt'
 
         # Make sure we can write to S3
         try:
             conn = _connect_s3(**credentials)
             bucket = conn.get_bucket(job.environment.s3_bucket, validate=False)
             k = _s3_key(bucket)
-            k.key = s3_state_key
+            k.key = s3_log_folder + 'status'
             k.set_contents_from_string('Pending')
         except:
             raise RuntimeError("Unable to start job. Could not write to S3 bucket: '%s'"
@@ -298,7 +306,9 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         serialized_job_file_path = _os.path.join(serialized_job_folder, 'job')
         job._serialize(serialized_job_file_path)
 
-        commander = Ec2ExecutionEnvironment._start_commander_host(job.environment, credentials)
+        commander = Ec2ExecutionEnvironment._start_commander_host(job.environment, credentials,
+                                                                  s3_log_folder)
+
         post_url = "http://%s:9004/submit" % commander.public_dns_name
         __LOGGER__.debug("Sending %s to %s" % (serialized_job_file_path, post_url))
 
@@ -307,13 +317,8 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
 
         # Technically the host should be started up now, but more than likely it needs a bit longer
         # before it's really started up.
-        MAX_TRIES, current_tries, response = 60, 0, None
-        post_params = {'s3_bucket': job.environment.s3_bucket,
-                  's3_state_key': s3_state_key, 
-                  's3_log_key': s3_log_key,
-                  's3_status_key': s3_status_key,
-                  'required_packages': job.get_required_packages()
-                  }
+        MAX_TRIES, current_tries, response = 240, 0, None
+        post_params = { 'required_packages': job.get_required_packages() }
         while current_tries <  MAX_TRIES:
             with open(serialized_job_file_path, 'rb') as job_file:
                 files = {'file': job_file}
@@ -347,7 +352,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
 
         result = _job.Ec2Job(job.name, job.tasks, job.environment, commander.public_dns_name,
                              commander.instance_id, job.environment.s3_bucket, s3_state_key,
-                             s3_log_key, s3_status_key)
+                             s3_log_folder, s3_log_folder + 'status-')
 
         if session is not None:
             session.save(result)
