@@ -5,7 +5,7 @@ import sys as _sys
 import tempfile as _tempfile
 import os as _os
 import pickle as _pickle
-
+import datetime as _datetime
 
 if __name__ == '__main__':
     if(len(_sys.argv) != 2):
@@ -20,11 +20,26 @@ if __name__ == '__main__':
         
         path = _os.path.dirname(serialized_job_location)
         job = _Job._deserialize(serialized_job_location)
+        # sets a file path for local job to update task status
+        status_path = _os.path.join(path, 'status')
+        starttime = str(_datetime.datetime.now().isoformat())
+
+        # write the start time of the job for local async 
+        try:
+            with open(_os.path.join(path, 'status'), 'w') as f:
+                f.writelines(str(starttime + "\n"))
+        except OSError:
+            pass
 
         # Now use the local launcher to run the job in this thread, will be async w.r.t other thread.
         runner = _LocalExecutionEnvironment()
-        runner.run_job(job, None)
+        runner.run_job(job, None, status_path=status_path)
 
+        # save end time to status
+        endtime = str(_datetime.datetime.now().isoformat())
+        with open(_os.path.join(path, 'status'), 'a+') as f:
+            f.writelines(str(endtime + "\n"))
+        
         # write out job metrics to file, so session process can incorporate into client job
         with open(_os.path.join(path, 'metrics'), 'w') as f:
             f.writelines(str(job.metrics))
