@@ -1,7 +1,6 @@
 import atexit
 from copy import copy
 import functools
-import json
 import logging as _logging
 import os
 from requests import get, post, ConnectionError
@@ -14,6 +13,8 @@ import boto.ec2.elb
 import boto.ec2.cloudwatch
 from boto.exception import BotoServerError
 import boto.iam.connection
+from urllib import quote_plus
+import json
 
 try:
     from subprocess import DEVNULL # py3k
@@ -57,7 +58,7 @@ class PredictiveServiceEnvironment:
         _logger.info("Notifying: %s" % host_name)
         data = json.dumps({'admin key': self.admin_key})
         url = 'http://%s:%d/control/poke' % (host_name, PredictiveServiceEnvironment._PORT_NUM)
-        self._post(url, data = data)
+        self._post(url, data=data)
 
     def __init__(self, admin_key):
         self.admin_key = admin_key
@@ -89,40 +90,8 @@ class PredictiveServiceEnvironment:
 
         return self.client_connection.query(po_name, method_params)
 
-    def get_metrics_url(self):
-        request_data = {'admin key': self.admin_key}
-        request_url = "%s://%s/control/cloudwatch_url" % (self.__get_schema(), self.load_balancer_dns_name)
-        resp = self._post(request_url, data = json.dumps(request_data))
-        return resp.text
-
-    def get_metrics(self, start_date=None, end_date=None):
-        if start_date:
-            assert(isinstance(start_date, datetime))
-
-        if end_date:
-            assert(isinstance(end_date, datetime))
-
-        if (start_date and not end_date) or (end_date and not start_date):
-            raise Exception('Getting metrics for a date range requires a '
-                            'start and end date')
-
-        request_data = {'admin key': self.admin_key}
-
-        if start_date:
-            if (start_date > datetime.utcnow()) or (end_date > datetime.utcnow()):
-                raise Exception('Date ranges may not extend past the current ' \
-                                'datetime')
-
-            request_data.update({'start_date': start_date.isoformat(),
-                                 'end_date': end_date.isoformat()})
-
-        request_url = "%s://%s/control/metrics" % (self.__get_schema(), self.load_balancer_dns_name)
-        resp = self._post(request_url, data = json.dumps(request_data))
-
-        try:
-            return resp.json()
-        except Exception as e:
-            raise RuntimeError("Cannot get metrics from service, error: %s" % resp.text)
+    def get_metrics_url(self, ps_name):
+        pass
 
     def _get_metrics(self, name, start_time, end_time, period):
         pass
@@ -134,14 +103,99 @@ class PredictiveServiceEnvironment:
         if not self._should_verify_certificate:
             self._should_verify_certificate = self.__should_verify_certificate()
 
-        headers = {'content-type': 'application/json' }
-        return post(url = url, data=data, headers=headers, verify=self._should_verify_certificate, timeout=10)
+        headers = {'content-type': 'application/json'}
+        return post(url=url, data=data, headers=headers,
+                    verify=self._should_verify_certificate, timeout=10)
+
+    @classmethod
+    def __make_url(cls, host_dns_name, endpoint):
+        return "http://%s:%d/%s" % (host_dns_name,
+                                    PredictiveServiceEnvironment._PORT_NUM,
+                                    endpoint)
+
+    def __cache_op_params(self, name, restart):
+        data = {"admin key": self.admin_key}
+
+        if name:
+            data.update({"name": name})
+
+        if restart:
+            data.update({"restart": restart})
+
+        return data
+
+    def __host_addrs(self):
+        if isinstance(self, Ec2PredictiveServiceEnvironment):
+            return [inst.public_dns_name for inst in self._get_all_hosts()]
+        else:
+            return ["localhost"]
+
+    def cache_enable(self, name, restart=True):
+        for host in self.__host_addrs():
+            try:
+                url = self.__make_url(host, "control/cache_enable")
+
+                params = self.__cache_op_params(name, restart)
+
+                self._post(url, json.dumps(params))
+            except Exception as e:
+                _logger.error('Could not enable cache on %s: %s' %
+                              (host, e.message))
+
+    def cache_disable(self, name):
+        for host in self.__host_addrs():
+            try:
+                url = self.__make_url(host, "control/cache_disable")
+
+                params = self.__cache_op_params(name, False)
+
+                self._post(url, json.dumps(params))
+            except Exception as e:
+                _logger.error('Could not disable cache on %s: %s' %
+                              (host, e.message))
+
+    def cache_clear(self, name):
+        for host in self.__host_addrs():
+            try:
+                url = self.__make_url(host, "control/cache_clear")
+
+                params = self.__cache_op_params(name, False)
+
+                self._post(url, json.dumps(params))
+            except Exception as e:
+                _logger.error('Could not clear cache on %s: %s' %
+                              (host, e.message))
+
+    def flush_logs(self):
+        for host in self.__host_addrs():
+            try:
+                url = self.__make_url(host, "control/flush_logs")
+                self._post(url, json.dumps({"admin key": self.admin_key}))
+            except Exception as e:
+                pass
+
+    def get_node_status(self, host_addr):
+        url = self.__make_url(host_addr, "control/status")
+        response = self._post(url, json.dumps(
+            {"admin key": self.admin_key}))
+        try:
+            _logger.info(response.text)
+            return json.loads(response.text)
+        except Exception as e:
+            return {"error": "Cannot get status for host %s, error: %s" %
+                    (host_addr, e.message)}
+
+    def get_service_status(self):
+        status = {}
+        for host in self.__host_addrs():
+            status.update({host: self.get_node_status(host)})
+        return status
 
     def _get_model_versions_for_host(self, host_dns_name):
         data = json.dumps({'admin key': self.admin_key})
         url = "http://%s:%d/control/list_objects" % (host_dns_name,
                                                      PredictiveServiceEnvironment._PORT_NUM)
-        resp = self._post(url, data = data)
+        resp = self._post(url, data=data)
         try:
             response = resp.text
             return json.loads(response)
@@ -174,7 +228,6 @@ class LocalPredictiveServiceEnvironment(PredictiveServiceEnvironment):
 
         self.certificate_is_self_signed = True
 
-
     def _get_state(self):
         result = {}
         result[ENV_TYPE] = 'LocalPredictiveServiceEnvironment'
@@ -183,12 +236,10 @@ class LocalPredictiveServiceEnvironment(PredictiveServiceEnvironment):
         result['admin_key'] = self.admin_key
         return result
 
-
     def get_status(self):
         return [ { 'id': 'localhost', 'dns_name': 'localhost', 'state':'InService',
                    'models': self._get_model_versions_for_host('localhost') } ]
 
-
     def terminate(self, delete_logs):
         _logger.info('Terminating service.')
         try:
@@ -439,13 +490,13 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
         min_period = 60 * int(delta_seconds / CW_MAX_DATA_POINTS / 60)
         if ((delta_seconds / CW_MAX_DATA_POINTS) % 60) > 0:
             min_period += 60
-       
+
         # set the min period if necessary
         if period < min_period:
             period = min_period
         if period < CW_MIN_PERIOD:
             period = CW_MIN_PERIOD
-        
+
         # get cloud watch handle
         cw = self._get_cloudwatch_handle()
 
@@ -466,6 +517,10 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
                 'num_keys_in_cache' : num_keys_in_cache}
 
 
+    def get_metrics_url(self, ps_name):
+        return 'https://console.aws.amazon.com/cloudwatch/home?region=' \
+               '%s#metrics:metricFilter=Pattern%%253D%s' % (self.region, quote_plus(ps_name))
+
     def _get_cloudwatch_handle(self):
         try:
             conn = boto.ec2.cloudwatch.connect_to_region(self.region, **self.aws_credentials)
@@ -581,13 +636,16 @@ class TestStubEnv(PredictiveServiceEnvironment):
         time_delta = timedelta(seconds=period)
         start = start_time
         while start < end_time:
-            num_requests.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'}) 
+            num_requests.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'})
             latency.append({'Average': 1.0, 'Timestamp': start, 'Unit':'Seconds'})
             num_healthy_hosts.append({'Average': 1.0, 'Timestamp': start, 'Unit':'Count'})
-            num_keys_in_cache.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'}) 
+            num_keys_in_cache.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'})
             start += time_delta
         return {'num_requests': num_requests, 'latency': latency, 'num_healthy_hosts': num_healthy_hosts,
                         'num_keys_in_cache': num_keys_in_cache}
 
+    def get_metrics_url(self, ps_name):
+        return 'test'
+
     def poke(self):
         pass
