@@ -10,6 +10,7 @@ import graphlab as _gl
 import inspect as _inspect
 import logging as _logging
 import tempfile as _tempfile
+import copy as _copy
 import datetime as _datetime
 import time as _time
 import os as _os
@@ -43,7 +44,7 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
                   :class:`~graphlab.deploy.environment.Hadoop` |
                   :class:`~graphlab.deploy.environment.LocalAsync`
         Optional environment for execution. This would commonly hold access
-        keys, launch locations etc.  Also included in the environment object is
+        keys, launch locations, etc.  Also included in the environment object is
         a dictionary for associated metadata to pass to the execution. Default
         is 'LocalAsync', which will have the execution occur in the background
         locally.
@@ -56,9 +57,8 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
 
         If the function returns a dict then it will be collated into the
         results. If the function returns something other than a dict, it will
-        be cast to an str and that will be collated into the results. See the
-        examples below for more information. See the example below for more
-        information.
+        be cast to a str and that will be collated into the results. See the
+        example below for more information.
 
     function_arguments : dict | list [ dict ] | :class:`~graphlab.SFrame`
         Arguments to pass to the specified function as kwargs. To run multiple
@@ -67,10 +67,9 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
         once. Each row of the SFrame will be used to invoke the function.
 
     required_packages : list [ str ] | set [ str ]
-        List of package requirements (same as disutils.requires) format for
-        packages required for running this Job. This is most useful to specify
-        any non-standard Python packages required to run the function
-        specified.
+        List of packages required for running this job. The list should be
+        specified in the same format as distutils.requires. This is useful
+        for jobs that depend on non-standard Python packages.
 
     Returns
     -------
@@ -181,6 +180,14 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
         if name is None or name == '':
             name = 'job-%s-%s-%s' % (function.__name__, environment.name, _time.time())
 
+        # Is this running in parallel?
+        num_task_per_step = 1
+        if isinstance(environment, _environment.EC2) and hasattr(environment, 'num_hosts'):
+            num_task_per_step = environment.num_hosts
+            temp_dir = "s3://%s/%s/temp-%s" % (environment.s3_bucket,
+                                               environment.s3_log_folder_path,
+                                               _time.time())
+
         combiner = _Task(name + '-combiner')
         combiner.set_code(_combiner_task)
         idx = -1
@@ -193,18 +200,34 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
             # validate that no GL data structures being passed in function_arguments
             for key, value in binding.items():
                 if _Task._is_valid_data_structure(value):
-                    raise RuntimeError("Validation Failed: Unsupported type for function_arguments. Function arguments must be basic types that can be serialized into JSON. Invalid function_argument: '%s', type: '%s'" % (key, type(value)))
+                    raise RuntimeError("Validation Failed: Unsupported type for function_arguments. "
+                                       "Function arguments must be basic types that can be "
+                                       "serialized into JSON. Invalid function_argument: '%s', "
+                                       "type: '%s'" % (key, type(value)))
 
             task.set_params({'params':binding, 'function':function})
-            task.set_outputs(['output'])
-
-            # create dependency for output from task to combiner task
-            combiner.set_inputs({'in-%d' % idx : (task, 'output')})
+            if num_task_per_step > 1:
+                task.set_params({'output_location': temp_dir + '/' + str(idx)})
+            else:
+                task.set_outputs(['output'])
+                # create dependency for output from task to combiner task
+                combiner.set_inputs({'in-%d' % idx : (task, 'output')})
             tasks.append(task)
             _gl.deploy.tasks.delete(task)
 
         combiner.set_params({'num' : len(bindings)})
-        tasks.append(combiner)
+
+        if num_task_per_step > 1:
+            # Turn this into a pipline.
+            tasks = _chunks(tasks, num_task_per_step)
+            combiner.set_params({'root_temp_output': temp_dir})
+            tasks.append([combiner])
+            temp = _Pipeline(name)
+            temp.set_tasks(tasks)
+            tasks = temp
+        else:
+            tasks.append(combiner)
+
         _gl.deploy.tasks.delete(combiner)
         tracker.track('deploy.job.create.fn', value=1)
 
@@ -228,7 +251,10 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
     # ensures the packages will be present on execution
     if required_packages is not None:
         packages = tasks[0].get_required_packages()
-        tasks[0].set_required_packages(packages | set(required_packages))
+        if isinstance(tasks[0], _Pipeline): 
+            tasks[0].steps[0][0].set_required_packages(packages | set(required_packages))
+        else: 
+            tasks[0].set_required_packages(packages | set(required_packages))
 
     if name is None or name == '':
         task = tasks[0]
@@ -256,12 +282,10 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
 
     # Create artifact from their names, if necessary. Clone all artifacts. Add any bindings.
     cloned_artifacts = []
-    using_pipeline = False
     for steps in tasks:
 
         # handle pipeline differently then task
         if isinstance(steps, _Pipeline):
-            using_pipeline = True
             binding = None
             if isinstance(steps, tuple):
                 (cur_artifact, binding) = steps
@@ -357,21 +381,8 @@ def _get_results_path(environment, job_name, folder_name = 'stored_results'):
     dirname = job_name
     results_path = _os.path.join(_tempfile.gettempdir(), dirname, folder_name)
     if isinstance(environment, _environment.EC2):
-
-        # get credentials
-        if not hasattr(environment, 'aws_access_key'):
-            # no access key in environment, try to get from get_credentials
-            try:
-                access_key, secret_key = _gl.aws.get_credentials()
-            except KeyError:
-                # not in get_credentials and not in environment, fail validation
-                raise RuntimeError('Validation Failed: Unable to retrieve AWS credentials for S3, needed for writing results from this execution. Use graphlab.aws.set_credentials() or set the credentials in the EC2 Environment object.')
-        else:
-            access_key = environment.aws_access_key
-            secret_key = environment.aws_secret_key
-
         path = _os.path.join(environment.s3_bucket, environment.s3_log_folder_path, dirname, folder_name)
-        results_path = "s3://%s:%s:%s" % (access_key, secret_key, path)
+        results_path = "s3://%s" % (path)
     elif isinstance(environment, _environment.Hadoop):
         # get the root path in hdfs land, to support actually launching in HDFS
         if environment.get_config_dir() is not None:
@@ -433,13 +444,37 @@ def _wrapper_task(task):
         end = _datetime.datetime.now()
         raise e
     finally:
-        task.outputs['output'] = _gl.SFrame({'input':[params], 'result':[result], 'start_time':[start], 'end_time':[end], 'status':[status], 'message':[message]})
+        output = _gl.SFrame({'input':[params], 'result':[result], 'start_time':[start],
+                             'end_time':[end], 'status':[status], 'message':[message]})
+        if 'output_location' not in task.params:
+            task.outputs['output'] = output
+        else:
+            # This is being run in a distributed way.
+            output_location = task.params['output_location']
+            print "saving output to temporary location: %s" % output_location
+            output.save(output_location)
+
 
 def _combiner_task(task):
     import graphlab as gl
-    print 'Combining %d tasks' % task.params['num']
+
+    num_intermediate_results = task.params['num']
+    print 'Combining %d tasks' % num_intermediate_results
+
+    def path_iter():
+        temp_root_path = task.params['root_temp_output']
+        for i in range(num_intermediate_results):
+            path = temp_root_path + '/' + str(i)
+            yield gl.SFrame(path)
+
+    if 'root_temp_output' not in task.params:
+        result_iter = task.inputs.itervalues()
+    else:
+        # Combining results from a distributed run.
+        result_iter = path_iter()
+
     results_list = []
-    for result in task.inputs.itervalues():
+    for result in result_iter:
         # iterate one row SFrame to get dict for each row
         for row in result:
             results_list.append(row)
@@ -520,3 +555,11 @@ def __validate_path_prefixes(task, prefix, validation_msgs):
         if value is not None and not str(value).strip().lower().startswith(prefix):
            validation_msgs.append("Validation error: Task: '%s', output named: '%s' not being saved to expected prefix: '%s', being saved to: '%s'" % (task.name, name, prefix, value))
 
+
+def _chunks(l, n):
+    '''
+    Breaks a list (parameter l) in n sized pieces.
+    '''
+    if n < 1:
+        n = 1
+    return [l[i:i + n] for i in range(0, len(l), n)]
