@@ -1,5 +1,18 @@
+"""
+This module hosts all the extension functions and classes created via SDK.
+
+The function :py:func:`ext_import` is used to import a toolkit module (shared library)
+into the workspace. The shared library can be directly imported
+from a remote source, e.g. http, s3, or hdfs.
+The imported module will be under namespace `graphlab.extensions`.
+
+Alternatively, if the shared library is local, it can be directly imported
+using the python import statement. Note that graphlab must be imported first.
+"""
+
 # This is a fake meta namespace which contains toolkit functions and toolkit
 # models implemented as extensions in C++
+
 import graphlab as _gl
 import types as _types
 from graphlab.util import make_internal_url
@@ -13,18 +26,18 @@ from graphlab.cython.context import debug_trace as cython_context
 
 # Now. a bit of magic hackery is going to happen to this module.
 # This module is going to be first imported as graphlab.extensions
-# After which, inside graphlab/__init__.py, sys['graphlab.extensions'] will 
+# After which, inside graphlab/__init__.py, sys['graphlab.extensions'] will
 # be modified to become a class called _extension_wrapper which redirects
 # getattr calls into this module.
 #
-# The reason for this wrapping is so that uses of functions in gl.extensions 
+# The reason for this wrapping is so that uses of functions in gl.extensions
 # (for instance)
 #
 #     import graphlab as gl
 #     gl.extensions._demo_addone(5)
 #
 # This will normally not work because gl.extensions._publish() was not called
-# hence _demo_addone will not be found. 
+# hence _demo_addone will not be found.
 #
 # By wrapping the extensions module in another class, we can redefine
 # __getattr__ on that class and have it force gl.extensions._publish() when
@@ -39,16 +52,17 @@ from graphlab.cython.context import debug_trace as cython_context
 # so that tab completion in ipython works.
 #
 # The result is that we need gl.extensions._publish() to publish into both
-# places.  
+# places.
 #  - the current module
 #  - the gl.extensions wrapper
 #
-# Then the metapath importer (this module) will just need to look in this 
+# Then the metapath importer (this module) will just need to look in this
 # module, breaking the recursive relation. And the gl.extensions wrapper will
 # have all the stuff in it for tab completion by IPython.
-#
+
 import sys as _sys
 _thismodule = _sys.modules[__name__]
+class_uid_to_class = {}
 
 def _wrap_function_return(val):
     """
@@ -62,6 +76,16 @@ def _wrap_function_return(val):
         return _gl.SFrame(_proxy = val)
     elif type(val) == _UnitySArrayProxy:
         return _gl.SArray(_proxy = val)
+    elif type(val) == _UnityModel:
+        # we need to cast it up to the appropriate type
+        try:
+            if '__uid__' in val.list_fields():
+                uid = val.get('__uid__')
+                if uid in class_uid_to_class:
+                    return class_uid_to_class[uid](_proxy=val)
+        except:
+            pass
+        return val
     elif type(val) == list:
         return [_wrap_function_return(i) for i in val]
     elif type(val) == dict:
@@ -92,12 +116,14 @@ def _translate_function_arguments(argument):
         return [_translate_function_arguments(i) for i in argument]
     elif type(argument) is dict:
         return {i:_translate_function_arguments(v) for (i, v) in argument.iteritems()}
+    elif hasattr(argument, '_tkclass') and hasattr(argument, '__glmeta__'):
+        return argument._tkclass
     else:
         return argument
 
 def _run_toolkit_function(fnname, arguments, args, kwargs):
     """
-    Dispatches arguments to a toolkit function. 
+    Dispatches arguments to a toolkit function.
 
     Parameters
     ----------
@@ -124,7 +150,7 @@ def _run_toolkit_function(fnname, arguments, args, kwargs):
     for i in range(len(args)):
         argument_dict[arguments[i]] = args[i]
 
-    # now fill with the kwargs. 
+    # now fill with the kwargs.
     for k in kwargs.keys():
         if k in argument_dict:
             raise TypeError("Got multiple values for keyword argument '" + k + "'")
@@ -150,10 +176,10 @@ def _run_toolkit_function(fnname, arguments, args, kwargs):
 def _make_injected_function(fn, arguments):
   return lambda *args, **kwargs: _run_toolkit_function(fn, arguments, args, kwargs)
 
-    
+
 class _ToolkitClass:
     """
-    The actual class class that is rewritten to become each user defined 
+    The actual class class that is rewritten to become each user defined
     toolkit class.
 
     Certain care with attributes (__getattr__ / __setattr__) has to be done to
@@ -185,7 +211,7 @@ class _ToolkitClass:
         except:
             raise _ToolkitError("Cannot create Toolkit Class for this class. "
                                "This class was not created with the new toolkit class system.")
-        
+
     def __dir__(self):
         return self._functions.keys() + self._get_properties + self._set_properties
 
@@ -203,7 +229,7 @@ class _ToolkitClass:
         for i in range(len(args)):
             argument_dict[arguments[i]] = args[i]
 
-        # now fill with the kwargs. 
+        # now fill with the kwargs.
         for k in kwargs.keys():
             if k in argument_dict:
                 raise TypeError("Got multiple values for keyword argument '" + k + "'")
@@ -242,7 +268,7 @@ class _ToolkitClass:
         else:
             raise AttributeError("no attribute " + name)
 
-def list_functions():
+def _list_functions():
     """
     Lists all the functions registered in unity_server.
     """
@@ -289,13 +315,13 @@ def _publish():
             mod = getattr(mod, path)
         _setattr_wrapper(mod, modpath[-1], newfunc)
 
-    # Repeat for classes 
+    # Repeat for classes
     tkclasslist = unity.list_toolkit_classes()
     for tkclass in tkclasslist:
         pathpos = fn.split('.')
         m = unity.describe_toolkit_class(tkclass)
         # of v2 type
-        if not ('functions' in m and 'get_properties' in m and 'set_properties' in m):
+        if not ('functions' in m and 'get_properties' in m and 'set_properties' in m and 'uid' in m):
             continue
 
         # create a new class
@@ -303,13 +329,14 @@ def _publish():
         # rewrite the init method to add the toolkit class name so it will
         # default construct correctly
 
-        new_class['__init__'] = _types.FunctionType(new_class['__init__'].func_code, 
+        new_class['__init__'] = _types.FunctionType(new_class['__init__'].func_code,
                                      new_class['__init__'].func_globals,
-                                     name='__init__', 
+                                     name='__init__',
                                      argdefs=(copy.deepcopy(tkclass), None),
-                                     closure=()) 
-        newclass = _types.ClassType(tkclass, (), new_class)
-        newclass.__dict__['__glmeta__'] = {'extension_name':fn}
+                                     closure=())
+        newclass = _types.ClassType(tkclass, (object,), new_class)
+        setattr(newclass, '__glmeta__', {'extension_name':fn})
+        class_uid_to_class[m['uid']] = newclass
         modpath = tkclass.split('.')
         # walk the module tree
         mod = _thismodule
@@ -355,14 +382,14 @@ class _ExtMetaPath(object):
         module_subpath = ".".join(fullname.split('.')[:-1])
         for path in sys.path:
             # joins the path to aaa/bbb/xxx
-            pathname = os.path.join(path, os.sep.join(fullname.split('.'))) 
+            pathname = os.path.join(path, os.sep.join(fullname.split('.')))
             # try to laod the ".so" extension
             try:
                 if os.path.exists(pathname + '.so'):
                     ext_import(pathname + '.so', module_subpath)
                     break
             except:
-                pass 
+                pass
 
             # try to laod the ".dylib" extension
             try:
@@ -370,7 +397,7 @@ class _ExtMetaPath(object):
                     ext_import(pathname + '.dylib', module_subpath)
                     break
             except:
-                pass 
+                pass
         ret = self.try_find_module(fullname, submodule_path)
         if ret is not None:
             return ret
@@ -413,7 +440,7 @@ class _ExtMetaPath(object):
         return mod
 
 _ext_meta_path_singleton = None
-    
+
 def _add_meta_path():
     """
     called on unity_server import to insert the meta path loader.
@@ -424,15 +451,21 @@ def _add_meta_path():
         _ext_meta_path_singleton = _ExtMetaPath()
         sys.meta_path += [_ext_meta_path_singleton]
 
+
 def ext_import(soname, module_subpath=""):
     """
-    Loads a graphlab toolkit module (a shared library) into the 
+    Loads a graphlab toolkit module (a shared library) into the
     gl.extensions namespace.
 
+    Toolkit module created via SDK can either be directly imported,
+    e.g. ``import example`` or via this function, e.g. ``graphlab.ext_import("example.so")``.
+    Use ``ext_import`` when you need more namespace control, or when
+    the shared library is not local, e.g. in http, s3 or hdfs.
+
     Parameters
     ----------
     soname : string
-        The filename of the shared library to load. 
+        The filename of the shared library to load.
         This can be a URL, or a HDFS location. For instance if soname is
         somewhere/outthere/toolkit.so
         The functions in toolkit.so will appear in gl.extensions.toolkit.*
@@ -440,19 +473,23 @@ def ext_import(soname, module_subpath=""):
     module_subpath : string, optional
         Any additional module paths to prepend to the toolkit module after
         it is imported. For instance if soname is
-        somewhere/outthere/toolkit.so, by default 
+        somewhere/outthere/toolkit.so, by default
         the functions in toolkit.so will appear in gl.extensions.toolkit.*.
         However, if I module_subpath="somewhere.outthere", the functions
         in toolkit.so will appear in gl.extensions.somewhere.outthere.toolkit.*
 
     Returns
     -------
-    Returns a list of functions and classes loaded.
+    out : a list of functions and classes loaded.
 
+    Examples
+    --------
     For instance, given a module which implements the function "square_root",
 
+    .. code-block:: c++
+
         #include <cmath>
-        #include <unity/lib/toolkit_function_macros.hpp>
+        #include <graphlab/sdk/toolkit_function_macros.hpp>
         double square_root(double a) {
           return sqrt(a);
         }
@@ -460,16 +497,17 @@ def ext_import(soname, module_subpath=""):
         BEGIN_FUNCTION_REGISTRATION
         REGISTER_FUNCTION(square_root, "a");
         END_FUNCTION_REGISTRATION
-    
+
     compiled into example.so
 
     >>> graphlab.ext_import('example1.so')
     ['example1.square_root']
+
     >>> graphlab.extensions.example1.square_root(9)
     3.0
 
     We can customize the import location with module_subpath which can be
-    used to avoid namespace conflicts when you have multiple toolkits with the 
+    used to avoid namespace conflicts when you have multiple toolkits with the
     same filename.
 
     >>> graphlab.ext_import('example1.so', 'math')
@@ -478,8 +516,9 @@ def ext_import(soname, module_subpath=""):
     3.0
 
     The module can also be imported directly, but graphlab *must* be imported
-    first. graphlab will intercept the module loading process to load the 
+    first. graphlab will intercept the module loading process to load the
     toolkit.
+
     >>> import graphlab
     >>> import example1 #searches for example1.so in all the python paths
     >>> example1.square_root(9)
@@ -534,7 +573,7 @@ class _Closure:
     arguments: An array of the same length as the toolkit native function.
         Each array element is an array of 2 elements [is_capture, value]
 
-        If is_capture == 1: 
+        If is_capture == 1:
             value contains the captured value
         If is_capture == 0:
             value contains a number denoting the lambda argument position.
@@ -542,16 +581,16 @@ class _Closure:
     Example:
         lambda x, y: fn(10, x, x, y)
 
-    Then arguments will be 
+    Then arguments will be
         [1, 10], -->  is captured value. has value 10
-        [0, 0],  -->  is not captured value. is argument 0 of the lambda. 
-        [0, 0],  -->  is not captured value. is argument 0 of the lambda. 
-        [0, 1]   -->  is not captured value. is argument 1 of the lambda. 
+        [0, 0],  -->  is not captured value. is argument 0 of the lambda.
+        [0, 0],  -->  is not captured value. is argument 0 of the lambda.
+        [0, 1]   -->  is not captured value. is argument 1 of the lambda.
     """
     def __init__(self, native_fn_name, arguments):
         self.native_fn_name = native_fn_name
         self.arguments = arguments
-        
+
 
 def _descend_namespace(caller_globals, name):
     """
@@ -593,7 +632,7 @@ def _build_native_function_call(fn):
         return _Closure(native_function_name, arglist)
 
     # ok. its not a native function
-    from graphlab_util.lambda_closure_capture import translate 
+    from graphlab_util.lambda_closure_capture import translate
     from graphlab_util.lambda_closure_capture import Parameter
 
     # Lets see if it is a simple lambda
@@ -604,7 +643,7 @@ def _build_native_function_call(fn):
     native_function_name = _get_toolkit_function_name_from_function(function)
     if native_function_name == "":
         raise RuntimeError("Lambda does not contain a native function")
-    
+
     argnames = _get_argument_list_from_toolkit_function_name(native_function_name)
 
     # ok. build up the argument list. this is mildly annoying due to the mix of
@@ -622,7 +661,7 @@ def _build_native_function_call(fn):
         else:
             # this is a captured value
             arglist[i] = [1, arg]
-    
+
     # now. the named arguments are somewhat annoying
     for i in capture.named_args:
         arg = capture.named_args[i]
