@@ -4,7 +4,6 @@
 This module defines the (internal) utility functions used by the toolkits.
 """
 
-from array import array as _array
 import json
 from graphlab.data_structures.sframe import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
@@ -60,14 +59,27 @@ def _SGraphFromJsonTree(json_str):
         sg = sg.add_edges(edges)
     return sg
 
-def _toolkit_repr_print(model, fields, width=20):
+class _precomputed_field(object):
+    def __init__(self, field):
+        self.field = field
+
+def _toolkit_repr_print(model, fields, width=20, section_titles=None):
     """
     Display a toolkit repr according to some simple rules.
 
-    @arg model : GraphLab model
-    @arg fields: List of tuples (display_name, field_name)
+    Parameters
+    ----------
+    model : GraphLab model
 
-    Example:
+    fields: List of lists of tuples
+        Each tuple should be (display_name, field_name), where field_name can
+        be a string or a _precomputed_field object.
+
+
+    section_titles: List of section titles, one per list in the fields arg.
+
+    Example
+    -------
 
         model_fields = [
             ("L1 penalty", 'l1_penalty'),
@@ -86,14 +98,32 @@ def _toolkit_repr_print(model, fields, width=20):
             ("Log-likelihood", 'training_loss')]
 
         fields = [model_fields, solver_fields, training_fields]:
+
+        section_titles = ['Model description',
+                          'Solver description',
+                          'Training information']
+
+        _toolkit_repr_print(model, fields, section_titles)
     """
     key_str = "{:<{}}: {}"
 
     ret = []
-    ret.append(key_str.format("Class", width, model.__class__.__name__))
-    for tranche_fields in fields:
-        for k, v in tranche_fields:
-            value = model.get(v)
+    ret.append(key_str.format("Class", width, model.__class__.__name__) + '\n')
+    if section_titles is not None:
+        assert len(section_titles) == len(fields), \
+        "The number of section titles ({0}) ".format(len(section_titles)) +\
+        "doesn't match the number of groups of fields, {0}.".format(len(fields))
+
+    for i in range(len(fields)):
+        if section_titles is not None:
+            ret.append(section_titles[i])
+            bar = '-' * len(section_titles[i])
+            ret.append(bar)
+        for k, v in fields[i]:
+            if isinstance(v, _precomputed_field):
+                value = v.field
+            else:
+                value = model.get(v)
             if isinstance(value, float):
                 try:
                     value = round(value, 4)
@@ -135,15 +165,16 @@ def _raise_error_if_column_exists(dataset, column_name = 'dataset',
     err_msg = 'The SFrame {0} must contain the column {1}.'.format(
                                                 dataset_variable_name,
                                              column_name_error_message_name)
-    if column_name not in dataset:
+    if column_name not in dataset.column_names():
       raise ToolkitError, str(err_msg)
 
 def _check_categorical_option_type(option_name, option_value, possible_values):
     """
     Check whether or not the requested option is one of the allowed values.
     """
-    err_msg = 'The provided value for {0} is not valid. '.format(option_name)
-    err_msg += '{0} should be changed to one of the following:'.format(option_value)
+    err_msg = '{0} is not a valid option for {0}. '.format(option_value, option_name)
+    err_msg += ' Expected one of: '.format(possible_values)
+
     err_msg += ', '.join(possible_values)
     if option_value not in possible_values:
         raise ToolkitError, err_msg
@@ -201,91 +232,6 @@ def _raise_error_evaluation_metric_is_valid(metric, allowed_metrics):
       raise ToolkitError, err_msg % (metric,
                           ', '.join(map(lambda x: "'%s'" % x, allowed_metrics)))
 
-def _validate_array_type(dataset, col_name):
-    """
-    Check that each element of the array contains an equal number of numeric types.
-    The check is performed on the first 100 or all rows of the column, whichever
-    is smaller.
-
-    Parameters
-    ----------
-    dataset: SFrame
-        Input dataset.
-
-    col_name: str
-        Name of the column to check.
-
-    Returns
-    -------
-    out: bool
-        If True, 'col_name' is a valid array-type feature.
-    """
-    is_valid = True
-    nsamples = min(dataset.num_rows(), 100)
-    m = 0
-    for i in range(nsamples):
-        if i == 0:
-            m = len(dataset[col_name][i])
-
-        if len(dataset[col_name][i]) != m:
-            is_valid = False
-            _logging.warning("Column " + col_name +
-                " is excluded as a feature because row " + str(i) +
-                " contains an array of a different length. All arrays in the " +
-                " column must have equal length.\n")
-            break
-
-        # Now verify that each element in the array is a number
-        for j, elem in enumerate(dataset[col_name][i]):
-            if type(elem) not in [float, int]:
-                is_valid = False
-                _logging.warning("Column " + col_name +
-                    " is excluded as a feature because row " + str(i) + 
-                    " element " + str(j) + "is an element of non-numeric type." +
-                    " Only numeric array elements (int or float) are accepted" +
-                    " as valid features.\n")
-                break
-
-        if not is_valid:
-            break
-
-    return is_valid
-
-def _validate_dict_type(dataset, col_name):
-    """
-    Check that each dictionary in the specified column contains only numeric values.
-    The check is performed on the first 100 or all rows of the column, whichever
-    is smaller.
-
-    Parameters
-    ----------
-    dataset: SFrame
-        Input dataset.
-
-    col_name: str
-        Name of the column to check.
-
-    Returns
-    -------
-    out: bool
-        If True, 'col_name' is a valid dict-type feature.
-    """
-    is_valid = True
-    nsamples = min(dataset.num_rows(), 100)
-    for i in range(nsamples):
-        for k, v in dataset[col_name][i].iteritems():
-            if type(v) not in [float, int]:
-                is_valid = False
-                _logging.warning("Column " + col_name +
-                    " is excluded as a feature because row " + str(i) + " key " + str(k) +
-                    " has a value of non-numeric type. Only numeric value types (int or float) are accepted as valid features.\n")
-                break
-
-        if not is_valid:
-            break
-
-    return is_valid
-
 def _select_valid_features(dataset, features, valid_feature_types,
                            target_column=None):
     """
@@ -349,25 +295,13 @@ def _select_valid_features(dataset, features, valid_feature_types,
                              " as a feature.")
 
         else:
-            include_flag = True     
-
-            if col_type_map[col_name] not in valid_feature_types:
-                include_flag = False
-
-            else:
-                if col_type_map.get(col_name) == _array:
-                    include_flag = _validate_array_type(dataset, col_name)
-                if col_type_map.get(col_name) == dict:
-                    include_flag = _validate_dict_type(dataset, col_name)
-
-            if include_flag is True:
+            if col_type_map[col_name] in valid_feature_types:
                 valid_features.append(col_name)
             else:
                 _logging.warning("Column " + col_name +
                     " is excluded as a feature due to invalid column type: " +
                     str(col_type_map.get(col_name)) + ".\n")
 
-
     if len(valid_features) == 0:
         raise ValueError("The dataset does not contain any valid feature columns. " +
             "Accepted feature types are " + str(valid_feature_types) + ".")
