@@ -1,7 +1,6 @@
 __all__ = ['NeuralNet', 'MultiLayerPerceptrons', 'ConvolutionNet']
 
 from . import layers as layers
-from . import _cxxnet_adaptor
 from .layers import _LayerBase, _layer_from_string, types
 from .. _main import ToolkitError as _ToolkitError
 from graphlab.util import make_internal_url as _make_internal_url
@@ -444,32 +443,6 @@ class NeuralNet(_generic.CommonEqualityMixin):
         paramstr += "## end network parameters\n"
         return netstr + paramstr
 
-    def _cxxnet_layer_str(self):
-        """ Returns the config string for cxxnet """
-        begin = "netconfig = start\n"
-        body = ""
-        current_layer = 0
-        for i in range(0, len(self._layers)):
-            # dropout and softmax is self-connected in cxxnet conf
-            if (type(self._layers[i]) in (layers.DropoutLayer, layers.SoftmaxLayer)):
-                layer_str = "layer[%d->%d] = %s" % (current_layer, current_layer, self._layers[i]._cxxnet_str())
-            else:
-                layer_str = "layer[%d->%d] = %s" % (current_layer, current_layer + 1, self._layers[i]._cxxnet_str())
-                current_layer += 1
-            body += layer_str
-        end = "netconfig = end\n"
-        netstr = begin + body + end
-        return netstr
-
-    def _cxxnet_param_str(self):
-        paramstr = "\n## learning parameters\n"
-        for k, v in self.params.iteritems():
-            k = _cxxnet_adaptor.gl_to_cxxnet_map.get(str(k), str(k))
-            v = _cxxnet_adaptor.gl_to_cxxnet_map.get(str(v), str(v))
-            paramstr += k + ' = ' + v + '\n'
-        paramstr += "## end learning parameters\n"
-        return paramstr
-
     def _load(self, url):
         """ Internal. Load the network from url. """
         conf_str = _gl_connect.get_unity().__read__(url)
@@ -557,12 +530,15 @@ class MultiLayerPerceptrons(NeuralNet):
         if num_hidden_layers != len(num_hidden_units):
             raise ValueError('Length of num_hidden_units must equal to num_hidden_layers')
         for i in range(0, num_hidden_layers - 1):
-            self._layers.append(layers.FullConnectionLayer(num_hidden_units[i], **kwargs))
-            self._layers.append(ActivationLayer(**kwargs))
+            self._layers.append(layers.FullConnectionLayer(num_hidden_units[i]))
+            self._layers[-1]._set_params(**kwargs)
+            self._layers.append(ActivationLayer())
+            self._layers[-1]._set_params(**kwargs)
 
         # output layer
         if (num_hidden_units[-1] > 0):
-            self._layers.append(layers.FullConnectionLayer(num_hidden_units[-1], **kwargs))
+            self._layers.append(layers.FullConnectionLayer(num_hidden_units[-1]))
+            self._layers[-1]._set_params(**kwargs)
             if (num_hidden_units[-1] > 1):
                 self._layers.append(layers.SoftmaxLayer())
 
@@ -666,15 +642,18 @@ class ConvolutionNet(NeuralNet):
         for n in range(num_convolution_layers):
             self._layers.append(layers.ConvolutionLayer(kernel_size=kernel_size,
                                                         num_channels=num_channels,
-                                                        stride=stride,
-                                                        **kwargs))
+                                                        stride=stride))
+            self._layers[-1]._set_params(**kwargs)
             self._layers.append(PoolingLayer(kernel_size=kernel_size,
-                                             stride=stride, **kwargs))
-        self._layers.append(layers.FlattenLayer(**kwargs))
+                                             stride=stride))
+            self._layers[-1]._set_params(**kwargs)
+        self._layers.append(layers.FlattenLayer())
+        self._layers[-1]._set_params(**kwargs)
 
         # output layer
         if (num_output_units > 0):
-            self._layers.append(layers.FullConnectionLayer(num_output_units, **kwargs))
+            self._layers.append(layers.FullConnectionLayer(num_output_units))
+            self._layers[-1]._set_params(**kwargs)
             if (num_output_units > 1):
                 self._layers.append(layers.SoftmaxLayer())
 
