@@ -110,7 +110,7 @@ def rmse(targets, predictions):
     return _graphlab.extensions._supervised_streaming_evaluator(targets, 
                                                           predictions, "rmse")
 
-def confusion_matrix(targets, predictions, threshold=0.5):
+def confusion_matrix(targets, predictions):
     r"""
     Compute the confusion matrix for classifier predictions.
 
@@ -126,7 +126,7 @@ def confusion_matrix(targets, predictions, threshold=0.5):
     Returns
     -------
     out : SFrame 
-        An SFrame containing counts for 'true_label', 'predicted_label' and 
+        An SFrame containing counts for 'target_label', 'predicted_label' and 
         'count' corresponding to each pair of true and predicted labels.
 
     See Also
@@ -149,8 +149,6 @@ def accuracy(targets, predictions):
     r"""
     Compute the proportion of correct predictions.
 
-    Predictions that are equal to `threshold` are counted as class 0.
-    
     Parameters
     ----------
     targets : SArray
