@@ -59,9 +59,10 @@ class ShortestPathModel(_ModelBase):
     | training_time  | Total training time of the model                           |
     +----------------+------------------------------------------------------------+
 
-    An instance of this model can be created using
-    :func:`graphlab.shortest_path.create`.  Do NOT construct the model
-    directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.shortest_path.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
 
     See Also
     --------
@@ -195,17 +196,18 @@ class ShortestPathModel(_ModelBase):
             g.edges[weight_field] = 1
 
         # Traverse the graph once and get the parent row id for each vertex
-        def traverse_fun(src, edge, dst):
-            if src['__id'] == source_vid:
-                src['__parent__'] = src['row_id']
-            if dst['distance'] == src['distance'] + edge[weight_field]:
-                dst['__parent__'] = max(dst['__parent__'], src['row_id'])
-            return (src, edge, dst)
-
+        # def traverse_fun(src, edge, dst):
+        #     if src['__id'] == source_vid:
+        #         src['__parent__'] = src['row_id']
+        #     if dst['distance'] == src['distance'] + edge[weight_field]:
+        #         dst['__parent__'] = max(dst['__parent__'], src['row_id'])
+        #     return (src, edge, dst)
+        #
         # the internal lambda appear to have some issues. 
-        # traverse_fun = lambda src, edge, dst:  \
-        #     gl.extensions._toolkits.graph.sssp.shortest_path_traverse_function(src, edge, dst, 
-        #             source_vid, weight_field)
+        import graphlab
+        traverse_fun = lambda src, edge, dst:  \
+            graphlab.extensions._toolkits.graph.sssp.shortest_path_traverse_function(src, edge, dst,
+                    source_vid, weight_field)
 
         g = g.triple_apply(traverse_fun, ['__parent__'])
         query_table = query_table.join(g.get_vertices()[['__id', '__parent__']], '__id').sort('row_id')
