@@ -259,13 +259,15 @@ def model_parameter_search(model_factory, train_set_path, save_path, test_set_pa
         >>> env = graphlab.deploy.environment.Local('local env')
 
         # For Kmeans using the default range of hyper-parameters for that model.
-        >>> model_parameter_search(env, graphlab.kmeans.create, train_file_path, '/data/results.gl')
+        >>> model_parameter_search(graphlab.kmeans.create, train_file_path, '/data/results.gl',
+                                   environment = env)
 
         # For Matrix Factorization with a target of 'rating', using the default range of hyper-params.
-        >>> model_parameter_search(env, graphlab.factorization_recommender.create, train_file_path,
-                                '/data/results.gl', test_set_path = test_file_path,
-                                standard_model_params = {'target': 'rating'}
-                              )
+        >>> model_parameter_search(graphlab.factorization_recommender.create, train_file_path,
+                                   '/data/results.gl', test_set_path = test_file_path,
+                                   standard_model_params = {'target': 'rating'},
+                                   environment = env
+                                  )
 
 
         # For boosted trees with a target of 'results', trying only a max_depth 5 and 10.
@@ -277,11 +279,12 @@ def model_parameter_search(model_factory, train_set_path, save_path, test_set_pa
 
         >>> graphlab.aws.set_credentials('RBZH792CTQPP7T435BGQ', '7x2hMqplWsLpU/qQCN6xAPKcmWo46TlPJXYTvKcv')
         >>> ec2_env = graphlab.deploy.environment.EC2('ec2 env', 's3://my-bucket-name/log-dir')
-        >>> job = model_parameter_search(ec2_env, graphlab.boosted_trees_classifier.create, train_file_path,
+        >>> job = model_parameter_search(graphlab.boosted_trees_classifier.create, train_file_path,
                                           's3://my-bucket-name/save_file_path',
                                           test_set_path = test_file_path,
                                           standard_model_params = {'target': 'result'},
-                                          hyper_params = {'max_depth': [5, 10]}
+                                          hyper_params = {'max_depth': [5, 10]},
+                                          environment = ec2_env
                                         )
 
         # Get the status of the jobs
