@@ -123,7 +123,8 @@ def create(observation_data,
 
         - *auto (default)*: automatically chooses the best solver for the data
                             and model parameters.
-        - *sgd*: Stochastic Gradient Descent
+        - *sgd*: Stochastic Gradient Descent.
+        - *als*: Alternating Least Squares.
 
     verbose : bool, optional
         Enables verbose output.
@@ -160,10 +161,18 @@ def create(observation_data,
     ...                                                user_data=user_info,
     ...                                                item_data=item_info)
 
+    **Using the Alternating Least Squares (ALS) solver**
+
+    The factorization model can also be solved using alternating least squares
+    (ALS) as a solver option.
+
+    >>> m3 = graphlab.factorization_recommender.create(sf, target='rating',
+                                                                solver = 'als')
 
     See Also
     --------
-    RankingFactorizationRecommender,  :class:`graphlab.recommender.ranking_factorization_recommender.RankingFactorizationRecommender`
+    RankingFactorizationRecommender,  
+    :class:`graphlab.recommender.ranking_factorization_recommender.RankingFactorizationRecommender`
     """
 
     _mt._get_metric_tracker().track('toolkit.recsys.factorization_recommender.create')
@@ -331,11 +340,16 @@ class FactorizationRecommender(_Recommender):
     :math:`\lambda_1` denotes the `linear_regularization` parameter and
     :math:`\lambda_2` the `regularization` parameter.
 
-    The model is trained using Stochastic Gradient Descent [sgd]_ with additional
-    tricks [Bottou]_ to improve convergence. The optimization is done in parallel
+    The model is trained using one of the following solvers:
+
+    (a) Stochastic Gradient Descent [sgd]_ with additional tricks [Bottou]_ 
+    to improve convergence. The optimization is done in parallel
     over multiple threads. This procedure is inherently random, so different
     calls to `create()` may return slightly different models, even with the
-    same `random_seed`.
+    same `random_seed`. 
+
+    (b) Alternating least squares (ALS), where the user latent factors are 
+    computed by fixing the item latent factors and vice versa.
 
     The Factorization Machine recommender model approximates target rating
     values as a weighted combination of user and item latent factors, biases,
