@@ -29,6 +29,7 @@ def create(observation_data,
            sgd_step_size=0,
            random_seed=0,
            binary_target = False,
+           solver = 'auto',
            verbose=True,
            **kwargs):
     """Create a RankingFactorizationRecommender that learns latent factors for each
@@ -101,6 +102,12 @@ def create(observation_data,
         items with a larger predicted score than this value. By default, the
         estimated 5% quantile is used (mean - 1.96*std_dev).
 
+    num_sampled_negative_examples : integer, optional
+        For each (user, item) pair in the data, the ranking sgd solver evaluates
+        this many randomly chosen unseen items for the negative example step.
+        Increasing this can give better performance at the expense of speed,
+        particularly when the number of items is large.  Default is 4.
+
     binary_target : boolean, optional
         Assume the target column is composed of 0's and 1's. If True, use
         logistic loss to fit the model.
@@ -121,6 +128,16 @@ def create(observation_data,
         unavoidable, so models trained with the same random seed may still
         differ. Default: 0.
 
+    solver : string, optional
+        Name of the solver to be used to solve the regression. See the
+        references for more detail on each solver. The available solvers for
+        this model are:
+
+        - *auto (default)*: automatically chooses the best solver for the data
+                            and model parameters.
+        - *ials*: Implicit matrix factorization [1]
+        - *sgd*: Stochastic Gradient Descent
+
     verbose : bool, optional
         Enables verbose output.
 
@@ -174,10 +191,26 @@ def create(observation_data,
     >>> m3 = ranking_factorization_recommender.create(sf, target='rating',
     ...                                               ranking_regularization = 0.1,
     ...                                               unobserved_rating_value = 1)
+    
+    **Using the implicit matrix factorization model**
+
+    Ranking factorization also implements implicit matrix factorization [1] as
+    an alternative model.
+
+    >>> m3 = ranking_factorization_recommender.create(sf, target='rating',
+                                                      solver = 'ials')
 
     See Also
     --------
-    FactorizationRecommender, :class:`graphlab.recommender.factorization_recommender.FactorizationRecommender`
+    :class:`graphlab.recommender.factorization_recommender.FactorizationRecommender`, 
+    :class:`graphlab.recommender.factorization_recommender.RankingFactorizationRecommender`
+
+    References
+    -----------
+
+    [1] Collaborative Filtering for Implicit Feedback Datasets Hu, Y.; Koren,
+        Y.; Volinsky, C. IEEE International Conference on Data Mining 
+        (ICDM 2008), IEEE (2008). 
 
     """
 
@@ -212,6 +245,7 @@ def create(observation_data,
             'ranking_regularization': ranking_regularization,
             'binary_target': binary_target,
             'max_iterations': max_iterations,
+            'solver' : solver,
             'sgd_step_size': sgd_step_size}
 
     if unobserved_rating_value is not None:
@@ -255,9 +289,11 @@ class RankingFactorizationRecommender(_Recommender):
 
     **Creating a RankingFactorizationRecommender**
 
-    This model can be created using
-    :func:`graphlab.recommender.ranking_factorization_recommender.create`.
-    Do NOT instantiate this model class directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.recommender.ranking_factorization_recommender.create` 
+    to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
     
     **Side information**
 
@@ -281,18 +317,11 @@ class RankingFactorizationRecommender(_Recommender):
     By default, RankingFactorizationRecommender optimizes for the
     precision-recall performance of recommendations.
 
-    When `ranking_regularization` is larger than zero, the model samples a small
-    set of unobserved user-item pairs and attempts to drive their rating
-    predictions below the value specified with `unobserved_rating_value`.
-    This has the effect of improving the precision-recall performance of
-    recommended items.
-
-
     **Model parameters**
 
     Trained model parameters may be accessed using
     `m.get('coefficients')` or equivalently `m['coefficients']`, where `m`
-    is a LinearRegressionRecommender.
+    is a RankingFactorizationRecommender.
 
     See Also
     --------
@@ -300,7 +329,8 @@ class RankingFactorizationRecommender(_Recommender):
 
     Notes
     -----
-    **Model Definition**
+
+    **Model Details**
 
     `RankingFactorizationRecommender` trains a model capable of predicting a score for
     each possible combination of users and items.  The internal coefficients of
@@ -423,6 +453,38 @@ class RankingFactorizationRecommender(_Recommender):
     `linear_side_features=True` performs better in terms of RMSE, but
     may require a longer training time.
 
+    num_sampled_negative_examples: For each (user, item) pair in the data, the
+    ranking sgd solver evaluates this many randomly chosen unseen items for the
+    negative example step.  Increasing this can give better performance at the
+    expense of speed, particularly when the number of items is large.
+
+
+    When `ranking_regularization` is larger than zero, the model samples
+    a small set of unobserved user-item pairs and attempts to drive their rating
+    predictions below the value specified with `unobserved_rating_value`.
+    This has the effect of improving the precision-recall performance of
+    recommended items.
+
+
+    ** Implicit Matrix Factorization**
+
+    `RankingFactorizationRecommender` had an additional option of optimizing
+    for ranking using the implcit matrix factorization model. The internal coefficients of
+    the model and its interpretation are identical to the model described above.
+    The difference between the two models is in the nature in which the objective
+    is achieved. Currently, this model does not incorporate any columns
+    beyond user/item (and rating) or side data.
+
+
+    The model works by transferring the raw observations (or weights) (r) into 
+    two separate magnitudes with distinct interpretations: preferences (p) and
+    confidence levels (c). The functional relationship between the weights (r)
+    and the confidence is either linear or logarithmic which can be toggled
+    by setting `ials_confidence_scaling_type` = `linear` (the default) or `log`
+    respectively. The rate of increase of the confidence with respect to the
+    weights is proportional to the `ials_confidence_scaling_factor` 
+    (default 1.0).
+
 
     Examples
     --------
@@ -439,6 +501,20 @@ class RankingFactorizationRecommender(_Recommender):
     ...                       'movie': ["a", "b", "c", "a", "b", "b", "c", "d"]})
     >>> m2 = graphlab.ranking_factorization_recommender.create(sf, 'user', 'movie')
 
+    **Implicit Matrix Factorization**
+
+    >>> sf = graphlab.SFrame({'user_id': ["0", "0", "0", "1", "1", "2", "2", "2"],
+    ...                       'item_id': ["a", "b", "c", "a", "b", "b", "c", "d"],
+    ...                       'rating': [1, 3, 2, 5, 4, 1, 4, 3]})
+    >>> m1 = graphlab.ranking_factorization_recommender.create(sf, target='rating', 
+                                                               solver='ials')
+
+    For implicit data, no target column is specified:
+
+    >>> sf = graphlab.SFrame({'user':  ["0", "0", "0", "1", "1", "2", "2", "2"],
+    ...                       'movie': ["a", "b", "c", "a", "b", "b", "c", "d"]})
+    >>> m2 = graphlab.ranking_factorization_recommender.create(sf, 'user', 'movie',
+                                                               solver='ials')
 
     **Including side features**
 
