@@ -51,10 +51,10 @@ class BoostedTreesRegression(_SupervisedLearningModel):
     tens of categories but is less suitable for highly sparse feautres such as
     text data.
 
-    An instance of this model can be created using
-    :py:func:`~graphlab.boosted_trees_regression.create`. 
-
-    Do NOT construct the model directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.boosted_trees_regression.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
 
     See Also
     --------
