@@ -293,10 +293,7 @@ class LinearRegression(_SupervisedLearningModel):
     """
     Linear regression is an approach for modeling a scalar target :math:`y` as
     a linear function of one or more explanatory variables denoted :math:`X`.
-    An instance of this model can be created using
-    :func:`graphlab.linear_regression.create`. Do not construct the model
-    directly.
-
+    
     Given a set of features :math:`x_i`, and a label :math:`y_i`, linear
     regression interprets the probability that the label is in one class as
     a linear function of a linear combination of the features.
@@ -318,6 +315,10 @@ class LinearRegression(_SupervisedLearningModel):
     where :math:`\lambda_1` is the ``l1_penalty`` and :math:`\lambda_2` is the
     ``l2_penalty``.
 
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.linear_regression.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
 
     Examples
     --------
