@@ -233,7 +233,13 @@ def create(dataset, target,
 
 class VowpalWabbitModel(_Model):
     """
-    Wrapper around Vowpal Wabbit.
+    Vowpal Wabbit Model wrapper.
+
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.vowpal_wabbit.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
+
     """
 
     def __init__(self, model_proxy):
