@@ -11,7 +11,6 @@ import os
 import platform
 import pprint
 import threading
-import time
 import uuid
 __ALL__ = [ 'MetricTracker' ]
 
@@ -27,16 +26,43 @@ METRICS_THREAD = None
 
 SHUTDOWN_MESSAGE = 'SHUTTING_DOWN'
 
+
 class _MetricsWorkerThread(threading.Thread):
   """Worker Thread for publishing metrics in the background."""
   
-  def __init__(self, tracker):
+  def __init__(self, mode, source):
     threading.Thread.__init__(self, name='metrics-worker')
-    global METRICS_QUEUE
-    self.tracker = tracker
+    
+    self._mode = mode
+    self._source = source
+
     self.queue = METRICS_QUEUE
     self.logger = logging.getLogger('graphlab.metrics')
-    self.startup_time = time.time()
+
+    self._tracker = None # librato metrics tracker
+    self._mixpanel = None # Mixpanel metrics tracker
+
+    buffer_size = 5
+    offline_buffer_size = 25
+    self._sys_info_set = False
+
+    self._usable = False
+    try:
+      if self._mode != 'PROD':
+        self.logger.info("Using MetricMock instead of real metrics, mode is: %s" % self._mode)
+        self._tracker = MetricMock()
+        self._mixpanel = MetricMock()
+      else:
+        self._tracker = librato.connect(CONFIG.librato_user, CONFIG.librato_token)
+        self._mixpanel = mixpanel.Mixpanel(CONFIG.mixpanel_user)
+    except Exception, e:
+      self.logger.warning("Unexpected exception connecting to Metrics service, disabling metrics, exception %s" % e)
+    else:
+      self._usable = True
+
+    self._distinct_id = 'unknown'
+    self._distinct_id = self._get_distinct_id()
+
 
   def run(self):
     while True:
@@ -44,186 +70,42 @@ class _MetricsWorkerThread(threading.Thread):
         metric = self.queue.get() # block until something received
 
         if (metric['event_name'] == SHUTDOWN_MESSAGE):
-          # shutting down, send total session time and then exit
-          self.logger.debug("Got shutdown command, sending session time and exiting")
-          now = time.time()
-          session_length_secs = now - self.startup_time
-          self.tracker._track('session.duration.secs', value=session_length_secs)
+          # shutting down
+          self.queue.task_done()
           break
       
-        self.tracker._track(metric['event_name'], metric['value'], metric['type'], metric['properties'], metric['meta'], metric['send_sys_info'])
-                
+        self._track(metric['event_name'], metric['value'], metric['type'], metric['properties'], metric['meta'], metric['send_sys_info'])
         self.queue.task_done()
 
-      except Queue.Empty:
-        # if queue is empty, wait try again
-        self.logger.debug("Nothing in Queue, doing nothing")
       except Exception as e:
-        self.logger.debug("Exception in processing metrics, %s" % e)
+        pass
 
-class MetricTracker:
-  def __init__(self, mode='UNIT', background_thread=True):
-    global METRICS_THREAD, METRICS_QUEUE
 
-    self._valid_modes = ['PROD']
-    self._tracker = None # librato metrics tracker
-    self._mixpanel = None # Mixpanel metrics tracker
-    self._url = 'https://metrics.graphlab.com'
-    self._usable = False
-
-    self._distinct_id = 'unknown'
-
-    # setup logging
-    self.logger = logging.getLogger('graphlab.metrics')
-
-    self._mode = mode
-
-    buffer_size = 5
-    offline_buffer_size = 25
-
-    try:
-      if self._mode not in self._valid_modes:
-        self.logger.info("Using MetricMock instead of real metrics, mode is: %s" % self._mode)
-        self._tracker = MetricMock()
-        self._mixpanel = MetricMock()
+  def _get_distinct_id(self):
+    if self._distinct_id == 'unknown':
+      poss_id = 'unknown'
+      id_file_path = "/".join([os.path.abspath(os.path.dirname(__file__)), '..','graphlab',"id"])
+      if os.path.isfile(id_file_path):
+        try:
+          with open(id_file_path, 'r') as f:
+            poss_id = f.readline()
+        except:
+          return "session-" + str(uuid.uuid4())
       else:
-        self._tracker = librato.connect(CONFIG.librato_user, CONFIG.librato_token)
-        self._mixpanel = mixpanel.Mixpanel(CONFIG.mixpanel_user,
-                                           OfflineConsumer(max_size=buffer_size,
-                                           offline_buffer_size=offline_buffer_size,
-                                           events_url=self._url+'/track',
-                                           people_url=self._url+'/engage'))
-    except Exception, e:
-      self.logger.warn("Unexpected exception connecting to Metrics service, disabling metrics, exception %s" % e)
+        # no distinct id found from installation,
+        # try to create one and write it to the appropriate location
+        # if not able to write to appropriate location, then create temp one
+        new_id = str(uuid.uuid4())
+        try:
+          with open(id_file_path, "w") as id_file:
+            id_file.write(new_id)
+        except:
+          return "session-" + str(uuid.uuid4())
+        return new_id
+      return poss_id.strip()
     else:
-      self._usable = True
+      return self._distinct_id
 
-    self._graphlab_version = CONFIG.version
-    self._distinct_id = self._get_distinct_id()
-    self._source = ("%s-%s" % (self._mode, self._graphlab_version))
-    self.logger.debug("Running with metric source: %s" % self._source)
-
-    self._sys_info_set = False
-
-    # background thread for metrics
-    self._queue = METRICS_QUEUE
-    self._thread = None
-    if background_thread:
-      self._start_queue_thread()
-
-  def __del__(self):
-    self.flush()
-    self._stop_queue_thread()
-
-  def track(self, event_name, value=1, type="gauge", properties={}, meta={}, send_sys_info=False):
-    """
-    Publishes event / metric to metrics providers.
-
-    This method is a facade / proxy, queuing up this metric for a background thread to process.
-    """
-    if self._mode != 'PROD' and (not (isinstance(value, int) or isinstance(value, float))):
-      raise Exception("Metrics attempted with value being not a number, unsupported.")
-
-    try:
-      item = dict(event_name=event_name, value=value, type=type, properties=properties, meta=meta, send_sys_info=send_sys_info)
-      self._queue.put_nowait(item) # don't wait if Queue is full, just silently ignore
-    except Queue.Full:
-      if not self._thread or not self._thread.is_alive():
-        self.logger.debug("Queue is full and background thread is no longer alive, trying to restart")
-        self._restart_queue_thread()
-      else:
-        self.logger.debug("Queue is full, doing nothing.")
-
-    except Exception as e:
-      self.logger.debug("Unexpected exception in queueing metrics, %s" % e)
-  
-  def _stop_queue_thread(self):
-    # send the shutting down message, wait for thread to exit
-    if self._thread is not None:
-      self.track(SHUTDOWN_MESSAGE)
-      self._thread.join(2.0)
-
-  def _start_queue_thread(self):
-    global METRICS_THREAD
-    if (self._thread is None):
-      self.logger.debug("Starting background thread")
-      self._thread = _MetricsWorkerThread(self)
-      METRICS_THREAD = self._thread
-      self._thread.daemon = True
-      self._thread.start()
-
-  def _restart_queue_thread(self):
-    global METRICS_THREAD
-
-    if (self._thread is not None and self._thread.is_alive()):
-      self._stop_queue_thread()
-      if self._thread.is_alive():
-        self.logger.debug("Background thread is still alive, after waiting 2s for it to die. Going to create new thread instead.")
-
-    METRICS_THREAD = None
-    del self._thread
-    self._thread = None
-
-    self._start_queue_thread()
-
-  def _track(self, event_name, value=1, type="gauge", properties={}, meta={}, send_sys_info=False):
-    """
-    Internal method to actually send metrics, expected to be called from background thread only.
-    """
-    if not self._usable:
-      return
-    the_properties = {}
-
-    if send_sys_info:
-      if not self._sys_info_set:
-        self._set_sys_info()
-      the_properties.update(self._sys_info)
-
-    the_properties.update(properties)
-
-    try:
-      # librato
-      self._tracker.submit(name=event_name, value=value, type="gauge", source=self._source, attributes=the_properties)
-    except Exception as e:
-      if (self._mode != 'PROD'):
-          self.logger.error("Error sending metrics to Librato, %s" % (e))
-          # This is happening either beacuse:
-          #   - The user is not connected to the internet
-          #   - The server is down (ours or Librato's ... whoever)
-          #   - Invalid url
-      pass
-
-    try:
-
-      # since mixpanel cannot send sizes or numbers, just tracks events, bucketize these here
-      if value != 1:
-          event_name = self._bucketize_mixpanel(event_name, value)
-          the_properties['value'] = value
-
-      # mixpanel
-      the_properties['source'] = self._source
-
-      self._mixpanel.track(self._distinct_id, event_name, properties=the_properties, meta=meta)
-
-    except Exception as e:
-      if (self._mode != 'PROD'):
-          self.logger.error("Error sending metrics to Mixpanel, %s" % (e))
-          # This is happening either beacuse:
-          #   - The user is not connected to the internet
-          #   - The server is down (ours or Mixpanel's ... whoever)
-          #   - Invalid url
-      pass
-
-  def flush(self):
-    if not self._usable:
-      return
-
-    try:
-      self.logger.debug("Flushing queued metrics")
-      self._tracker.submit()
-      self._mixpanel._consumer.flush()
-    except:
-      pass
 
   @staticmethod
   def _get_bucket_name_suffix(buckets, value):
@@ -242,7 +124,8 @@ class MetricTracker:
         suffix = '%d+' % buckets[-1]
     return suffix
 
-  def _bucketize_mixpanel(self, event_name, value):
+  @staticmethod
+  def _bucketize_mixpanel(event_name, value):
     """
     Take events that we would like to bucketize and bucketize them before sending to mixpanel
 
@@ -264,12 +147,11 @@ class MetricTracker:
     for (event_suffix, buckets) in bucket_events.iteritems():
         if event_name.endswith(event_suffix):
             # if the suffix matches one we expect, bucketize using the buckets defined above
-            return '%s.%s' % (event_name, MetricTracker._get_bucket_name_suffix(buckets, value))
+            return '%s.%s' % (event_name, _MetricsWorkerThread._get_bucket_name_suffix(buckets, value))
 
     # if there was no suffix match, just use the original event name
     return event_name
 
-
   def _set_sys_info(self):
     # Don't do this if system info has been set
     if self._sys_info_set:
@@ -323,59 +205,113 @@ class MetricTracker:
           tmp_list.append(str(t))
         except:
           pass
-
     return " ".join(tmp_list)
 
-  def _get_distinct_id(self):
-    if self._distinct_id == 'unknown':
-      poss_id = 'unknown'
-      id_file_path = "/".join([os.path.abspath(os.path.dirname(__file__)), '..','graphlab',"id"])
-      if os.path.isfile(id_file_path):
-        try:
-          with open(id_file_path, 'r') as f:
-            poss_id = f.readline()
-        except:
-          return "session-" + str(uuid.uuid4())
+
+  def _track(self, event_name, value=1, type="gauge", properties={}, meta={}, send_sys_info=False):
+    """
+    Internal method to actually send metrics, expected to be called from background thread only.
+    """
+    if not self._usable:
+      return
+    the_properties = {}
+
+    if send_sys_info:
+      if not self._sys_info_set:
+        self._set_sys_info()
+      the_properties.update(self._sys_info)
+
+    the_properties.update(properties)
+
+    try:
+      # librato
+      self._tracker.submit(name=event_name, value=value, type="gauge", source=self._source, attributes=the_properties)
+    except Exception as e:
+      pass
+
+    try:
+      # since mixpanel cannot send sizes or numbers, just tracks events, bucketize these here
+      if value != 1:
+          event_name = self._bucketize_mixpanel(event_name, value)
+          the_properties['value'] = value
+
+      # mixpanel
+      the_properties['source'] = self._source
+
+      self._mixpanel.track(self._distinct_id, event_name, properties=the_properties, meta=meta)
+    except Exception as e:
+      pass
+
+
+class MetricTracker:
+  def __init__(self, mode='UNIT', background_thread=True):
+    # setup logging
+    self.logger = logging.getLogger('graphlab.metrics')
+
+    self._mode = mode
+    self._queue = METRICS_QUEUE
+
+    self._source = ("%s-%s" % (self._mode, CONFIG.version))
+    self.logger.debug("Running with metric source: %s" % self._source)
+
+    # background thread for metrics
+    self._thread = None
+    if background_thread:
+      self._start_queue_thread()
+
+  def __del__(self):
+    try:
+      self._stop_queue_thread()
+    except:
+      # Lot of strange exceptions can happen when destructing, not really anything we can do...
+      pass
+
+  def _stop_queue_thread(self):
+    # send the shutting down message, wait for thread to exit
+    if self._thread is not None:
+      self.track(SHUTDOWN_MESSAGE)
+      self._thread.join(2.0)
+
+
+  def track(self, event_name, value=1, type="gauge", properties={}, meta={}, send_sys_info=False):
+    """
+    Publishes event / metric to metrics providers.
+
+    This method is a facade / proxy, queuing up this metric for a background thread to process.
+    """
+    if self._mode != 'PROD' and (not (isinstance(value, int) or isinstance(value, float))):
+      raise Exception("Metrics attempted with value being not a number, unsupported.")
+
+    try:
+      item = dict(event_name=event_name, value=value, type=type, properties=properties, meta=meta, send_sys_info=send_sys_info)
+      self._queue.put_nowait(item) # don't wait if Queue is full, just silently ignore
+    except Queue.Full:
+      if not self._thread or not self._thread.is_alive():
+        self.logger.debug("Queue is full and background thread is no longer alive, trying to restart")
+        self._restart_queue_thread()
       else:
-        # no distinct id found from installation,
-        # try to create one and write it to the appropriate location
-        # if not able to write to appropriate location, then create temp one
-        new_id = str(uuid.uuid4())
-        try:
-          with open(id_file_path, "w") as id_file:
-            id_file.write(new_id)
-        except:
-          return "session-" + str(uuid.uuid4())
-        return new_id
-      return poss_id.strip()
-    else:
-      return self._distinct_id
+        self.logger.debug("Queue is full, doing nothing.")
 
-class OfflineConsumer(mixpanel.BufferedConsumer):
-  def __init__(self, max_size, events_url, people_url, offline_buffer_size=25):
+    except Exception as e:
+      self.logger.debug("Unexpected exception in queueing metrics, %s" % e)
 
-    self._offline_buffer_size = offline_buffer_size
+  def _start_queue_thread(self):
+    global METRICS_THREAD
+    if (self._thread is None):
+      self.logger.debug("Starting background thread")
+      self._thread = _MetricsWorkerThread(self._mode, self._source)
+      METRICS_THREAD = self._thread
+      self._thread.daemon = True
+      self._thread.start()
 
-    if offline_buffer_size < max_size:
-      self._offline_buffer_size = max_size
+  def _restart_queue_thread(self):
+    global METRICS_THREAD
 
-    super(OfflineConsumer, self).__init__(max_size=max_size, events_url=events_url, people_url=people_url)
+    if (self._thread is not None and self._thread.is_alive()):
+      self._stop_queue_thread()
 
-  def _flush_endpoint(self, endpoint):
-    buf = self._buffers[endpoint]
-    while buf:
-      batch = buf[:self._max_size]
-      batch_json = '[{0}]'.format(','.join(batch))
-      try:
-        self._consumer.send(endpoint, batch_json)
-      except Exception, e:
-        if len(buf) < self._offline_buffer_size:
-          # Preserve the contents of the buffer
-          break
-        else:
-          # Clear the buffer
-          buf = buf[self._offline_buffer_size:]
-          break
-      buf = buf[self._max_size:]
+    METRICS_THREAD = None
+    del self._thread
+    self._thread = None
 
-    self._buffers[endpoint] = buf
+    self._start_queue_thread()
